// A headphone indicator for Gnome-shell
// Copyright (C) 2012 Meng Zhuo, Gnome-shell headphone Indicator extension contributors
// 
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.


const Main = imports.ui.main;
const PanelMenu = imports.ui.panelMenu;
const Lang = imports.lang;
const St = imports.gi.St;
const Gtk = imports.gi.Gtk;
const Extension = imports.misc.extensionUtils.getCurrentExtension();
const Jack = Extension.imports.jack;
const DBusIFace = Extension.imports.dbus;

const Status = {
    NONEXIST: 0,
    IN: 1,
    OUT: 2,
    UNKNOWN: 3
};

const Indicator = new Lang.Class({
    
    Name: 'HeadPhoneIndicator',
    
    Extends: PanelMenu.Button,
    
    _init : function()
    {
        this.parent(St.Align.START);
        
        this._jack = new Jack.HeadPhoneJack();
        
        this.actor.hide();
        
        if (this._jack.status != Status.NONEXIST){// We have sound card!
            //connect to change
            this._jack.connect('status-changed',Lang.bind(this,this._statusChanged));
            
            //connect to dbus 
            this._player = new DBusIFace.Media2Server();
  
            //icon stuff
            let iconTheme = Gtk.IconTheme.get_default();
            
            if (!iconTheme.has_icon('headphone-indicator'))
                iconTheme.append_search_path (Extension.dir.get_path());
                        
            this._icon = new St.Icon({ icon_type: St.IconType.SYMBOLIC,
                                        style_class: 'popup-menu-icon',
                                        icon_name: 'headphone-indicator',
                                        });
            
            
            this.actor.add_actor(this._icon);
            
            if (this._jack.status == Status.IN){
            
                this.actor.show();
            }

            this._controlFlag = false;

        }
        
        this.connect('destroy', Lang.bind(this, this._onDestroy));
        
    },
    _statusChanged : function ()
    {

        if (this._jack.status == Status.IN){
            
            this.actor.show();
            
            if (this._player.getPlaybackStatus() == 'Paused' && this._controlFlag){
                
                this._player.play();
                
            }
        }
        else{
            
            this.actor.hide();
            
            if (this._player.getPlaybackStatus() == 'Playing'){
                this._player.pause();
                this._controlFlag = true;
            }

        }
        return true;
    },
    destroy : function () {
        this._onDestroy();
    },
    _onDestroy: function() {
        this._jack.destroy();
        this._mediaServer.destroy();
    }
    

});

let indicator;

function enable() {
    if (!indicator) {
        indicator = new Indicator();
        Main.panel.addToStatusArea('HeadPhoneIndicator', indicator);
    }
}

function disable() {
    if (indicator) {
        indicator.destroy();
        indicator = null;
    }
}
function init() {}
