// A headphone indicator for Gnome-shell
// Copyright (C) 2012 Meng Zhuo, Gnome-shell headphone Indicator extension contributors
//
// This is a jack detection class using ALSA as its backend
// 
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

const GLib = imports.gi.GLib;
const Lang = imports.lang;
const Signals = imports.signals;
const Mainloop = imports.mainloop;

const HIGH_SENSITIVE_TIMEOUT = 250;
const LOW_SENSITIVE_TIMEOUT = 1000; //pulgged is not an urgent task. 

const Status = {
    NONEXIST: 0,
    IN: 1,
    OUT: 2,
    UNKNOWN: 3
};

const CardInfoCommandLine = 'amixer -c0 contents'; // FIX on card 1 since most of laptop has only one sound card.

const HeadPhoneJack = new Lang.Class({
    
    Name: 'HeadPhoneJack',
    
    _init : function ()
    {   
    
        this.connect('destroy', Lang.bind(this, this._onDestroy));
               
        this._oldStatus = this.status = Status.UNKNOWN;
        this._jackNumIDList = [];
        try {
            //find HeadPhone Jack numid;
            this._cardDevices = GLib.spawn_command_line_sync(CardInfoCommandLine)[1].toString().split('num');
            // if there is no sound exist, it will break ;
            let content = null;
            
            for (let i in this._cardDevices){
                
                if ( this._cardDevices[i].indexOf("Headphone Jack") > 0 ){ 
                    
                    //using indexOf to make speed up http://jsperf.com/regexp-vs-indexof
                    
                    content = this._cardDevices[i].split(',')[0];
                    
                    if (content != ''){ //remove none item
                        this._jackNumIDList.push('amixer cget num%s'.format(content));
                        //made those command line in array
                    }
                
                }
                
            }
            
            delete this._cardDevices; // save some RAM 
            

            
            this._update();
            
            this._mainLoopID = Mainloop.timeout_add(1000, Lang.bind(this, this._update)); 
//               It consume about 10% CPU(1.2 GHZ) if set the interval < 500,
//               It will be better if we have d-bus notify
            
        }
        catch(e) {
            global.logError('HeadPhone: '+ e.message);
            this.status = Status.NONEXIST;
        }
        
    },
    
    /******************************** Update invoked by timer ******************************/
	_update : function (){
	    
	    this.status = Status.OUT; //First guess	    
	    
        for( let i in this._jackNumIDList){
            
            if ( GLib.spawn_command_line_sync(this._jackNumIDList[i])[1].toString().indexOf('=on') >= 0 ){
                this.status = Status.IN;
                break; //there is no different if more than 1 jack plugged.
            }
            
        }
        
        if (this._oldStatus != this.status){
        
            this._changeLoopID();
            this._oldStatus = this.status;
            this.emit('status-changed');
            
        }
        
        return true; // for mainloop
    },
    _changeLoopID : function (){
    
        this._loopInterval = (this.status == Status.IN)?HIGH_SENSITIVE_TIMEOUT:LOW_SENSITIVE_TIMEOUT;
        Mainloop.source_remove(this._mainLoopID);
        this._mainLoopID = Mainloop.timeout_add(this._loopInterval, Lang.bind(this, this._update)); 
        
    },
    destroy : function (){
        this._onDestroy();
    },
    _onDestroy : function()
    {
        Mainloop.source_remove(this._mainLoopID);
    }
});
Signals.addSignalMethods(HeadPhoneJack.prototype);
